@@ -38,6 +38,7 @@
 #include <KSharedConfig>
 #include <KStandardAction>
 #include <KToggleAction>
+#include <KToolBarPopupAction>
 
 #include "kactioncollection.h"
 #include "kedittoolbar.h"
@@ -46,6 +47,8 @@
 
 #include "ktoolbarhelper_p.h"
 
+#include <algorithm>
+
 /*
  Toolbar settings (e.g. icon size or toolButtonStyle)
  =====================================================
@@ -81,9 +84,6 @@ public:
     KToolBarPrivate(KToolBar *qq)
         : q(qq)
         , isMainToolBar(false)
-#if KXMLGUI_BUILD_DEPRECATED_SINCE(5, 0)
-        , enableContext(true)
-#endif
         , unlockedMovable(true)
         , contextOrient(nullptr)
         , contextMode(nullptr)
@@ -118,7 +118,7 @@ public:
     void slotContextText();
     void slotContextTextRight();
     void slotContextTextUnder();
-    void slotContextIconSize();
+    void slotContextIconSize(QAction *action);
     void slotLockToolBars(bool lock);
 
     void init(bool readConfig = true, bool isMainToolBar = false);
@@ -138,9 +138,6 @@ public:
 
     KToolBar *const q;
     bool isMainToolBar : 1;
-#if KXMLGUI_BUILD_DEPRECATED_SINCE(5, 0)
-    bool enableContext : 1;
-#endif
     bool unlockedMovable : 1;
     static bool s_editable;
     static bool s_locked;
@@ -163,7 +160,13 @@ public:
     QAction *contextText;
     QAction *contextTextUnder;
     KToggleAction *contextLockAction;
-    QMap<QAction *, int> contextIconSizes;
+
+    struct ContextIconInfo {
+        QAction *iconAction = nullptr;
+        int iconSize = 0;
+    };
+
+    std::vector<ContextIconInfo> m_contextIconSizes;
 
     class IntSetting
     {
@@ -347,12 +350,11 @@ QMenu *KToolBarPrivate::contextMenu(const QPoint &globalPos)
 
         contextSize = new QMenu(i18n("Icon Size"), context);
 
-        auto contextIconSizeFunc = [this]() {
-            slotContextIconSize();
-        };
-
-        auto *act = contextSize->addAction(i18nc("@item:inmenu Icon size", "Default"), q, contextIconSizeFunc);
-        contextIconSizes.insert(act, iconSizeSettings.defaultValue());
+        auto *act = contextSize->addAction(i18nc("@item:inmenu Icon size", "Default"));
+        q->connect(act, &QAction::triggered, q, [this, act]() {
+            slotContextIconSize(act);
+        });
+        m_contextIconSizes.push_back({act, iconSizeSettings.defaultValue()});
 
         // Query the current theme for available sizes
         KIconTheme *theme = KIconLoader::global()->theme();
@@ -377,8 +379,11 @@ QMenu *KToolBarPrivate::contextMenu(const QPoint &globalPos)
                     text = i18n("Huge (%1x%2)", it, it);
                 }
 
-                // save the size in the contextIconSizes map
-                contextIconSizes.insert(contextSize->addAction(text, q, contextIconSizeFunc), it);
+                auto *act = contextSize->addAction(text);
+                q->connect(act, &QAction::triggered, q, [this, act]() {
+                    slotContextIconSize(act);
+                });
+                m_contextIconSizes.push_back({act, it});
             }
         } else {
             // Scalable icons.
@@ -398,8 +403,11 @@ QMenu *KToolBarPrivate::contextMenu(const QPoint &globalPos)
                             text = i18n("Huge (%1x%2)", it, it);
                         }
 
-                        // save the size in the contextIconSizes map
-                        contextIconSizes.insert(contextSize->addAction(text, q, contextIconSizeFunc), it);
+                        auto *act = contextSize->addAction(text);
+                        q->connect(act, &QAction::triggered, q, [this, act]() {
+                            slotContextIconSize(act);
+                        });
+                        m_contextIconSizes.push_back({act, it});
                         break;
                     }
                 }
@@ -452,7 +460,15 @@ QMenu *KToolBarPrivate::contextMenu(const QPoint &globalPos)
     // (no need to wait for the menu to open or aim at the arrow)
     if (auto *contextToolButton = qobject_cast<QToolButton *>(q->widgetForAction(contextButtonAction))) {
         if (contextToolButton->popupMode() == QToolButton::DelayedPopup || contextToolButton->popupMode() == QToolButton::MenuButtonPopup) {
-            if (auto *actionMenu = contextButtonAction->menu()) {
+            auto *actionMenu = contextButtonAction->menu();
+
+            if (!actionMenu) {
+                if (auto *toolBarPopupAction = qobject_cast<KToolBarPopupAction *>(contextButtonAction)) {
+                    actionMenu = toolBarPopupAction->popupMenu();
+                }
+            }
+
+            if (actionMenu) {
                 // In case it is populated on demand
                 Q_EMIT actionMenu->aboutToShow();
 
@@ -567,7 +583,7 @@ void KToolBarPrivate::slotAppearanceChanged()
 
 Qt::ToolButtonStyle KToolBarPrivate::toolButtonStyleSetting()
 {
-    KConfigGroup group(KSharedConfig::openConfig(), "Toolbar style");
+    KConfigGroup group(KSharedConfig::openConfig(), QStringLiteral("Toolbar style"));
     const QString fallback = KToolBarPrivate::toolButtonStyleToString(Qt::ToolButtonTextBesideIcon);
     return KToolBarPrivate::toolButtonStyleFromString(group.readEntry("ToolButtonStyle", fallback));
 }
@@ -595,7 +611,7 @@ void KToolBarPrivate::loadKDESettings()
         }
         **/
 
-        KConfigGroup group(KSharedConfig::openConfig(), "Toolbar style");
+        KConfigGroup group(KSharedConfig::openConfig(), QStringLiteral("Toolbar style"));
         const QString value = group.readEntry("ToolButtonStyleOtherToolbars", fallBack);
         toolButtonStyleSettings[Level_KDEDefault] = KToolBarPrivate::toolButtonStyleFromString(value);
     }
@@ -646,11 +662,11 @@ void KToolBarPrivate::slotContextAboutToShow()
 
     // try to find "configure toolbars" action
     QAction *configureAction = nullptr;
-    const char *actionName = KStandardAction::name(KStandardAction::ConfigureToolbars);
-    configureAction = findAction(QLatin1String(actionName));
+    const QString actionName = KStandardAction::name(KStandardAction::ConfigureToolbars);
+    configureAction = findAction(actionName);
 
     if (!configureAction && kmw) {
-        configureAction = kmw->actionCollection()->action(QLatin1String(actionName));
+        configureAction = kmw->actionCollection()->action(actionName);
     }
 
     if (configureAction) {
@@ -663,12 +679,18 @@ void KToolBarPrivate::slotContextAboutToShow()
         kmw->setupToolbarMenuActions();
         // Only allow hiding a toolbar if the action is also plugged somewhere else (e.g. menubar)
         QAction *tbAction = kmw->toolBarMenuAction();
-        if (!q->toolBarsLocked() && tbAction && !tbAction->associatedWidgets().isEmpty()) {
-            context->addAction(tbAction);
+        if (!q->toolBarsLocked() && tbAction) {
+            const QList<QObject *> associatedObjects = tbAction->associatedObjects();
+            const bool hasAssociatedWidgets = std::any_of(associatedObjects.cbegin(), associatedObjects.cend(), [](QObject *object) {
+                return (qobject_cast<QWidget *>(object) != nullptr);
+            });
+            if (hasAssociatedWidgets) {
+                context->addAction(tbAction);
+            }
         }
     }
 
-    KEditToolBar::setGlobalDefaultToolBar(q->QObject::objectName().toLatin1().constData());
+    KEditToolBar::setGlobalDefaultToolBar(q->QObject::objectName());
 
     // Check the actions that should be checked
     switch (q->toolButtonStyle()) {
@@ -687,13 +709,11 @@ void KToolBarPrivate::slotContextAboutToShow()
         break;
     }
 
-    QMapIterator<QAction *, int> it = contextIconSizes;
-    while (it.hasNext()) {
-        it.next();
-        if (it.value() == q->iconSize().width()) {
-            it.key()->setChecked(true);
-            break;
-        }
+    auto it = std::find_if(m_contextIconSizes.cbegin(), m_contextIconSizes.cend(), [this](const ContextIconInfo &info) {
+        return info.iconSize == q->iconSize().width();
+    });
+    if (it != m_contextIconSizes.cend()) {
+        it->iconAction->setChecked(true);
     }
 
     switch (q->mainWindow()->toolBarArea(q)) {
@@ -727,19 +747,24 @@ void KToolBarPrivate::slotContextAboutToHide()
     // We have to unplug whatever slotContextAboutToShow plugged into the menu.
     // Unplug the toolbar menu action
     KXmlGuiWindow *kmw = qobject_cast<KXmlGuiWindow *>(q->mainWindow());
-    if (kmw && kmw->toolBarMenuAction()) {
-        if (kmw->toolBarMenuAction()->associatedWidgets().count() > 1) {
-            context->removeAction(kmw->toolBarMenuAction());
+    if (kmw) {
+        QAction *tbAction = kmw->toolBarMenuAction();
+        const QList<QObject *> associatedObjects = tbAction->associatedObjects();
+        const int associatedWidgetsCount = std::count_if(associatedObjects.cbegin(), associatedObjects.cend(), [](QObject *object) {
+            return (qobject_cast<QWidget *>(object) != nullptr);
+        });
+        if (associatedWidgetsCount > 1) {
+            context->removeAction(tbAction);
         }
     }
 
     // Unplug the configure toolbars action too, since it's afterwards anyway
     QAction *configureAction = nullptr;
-    const char *actionName = KStandardAction::name(KStandardAction::ConfigureToolbars);
-    configureAction = findAction(QLatin1String(actionName));
+    const QString actionName = KStandardAction::name(KStandardAction::ConfigureToolbars);
+    configureAction = findAction(actionName);
 
     if (!configureAction && kmw) {
-        configureAction = kmw->actionCollection()->action(QLatin1String(actionName));
+        configureAction = kmw->actionCollection()->action(actionName);
     }
 
     if (configureAction) {
@@ -823,12 +848,15 @@ void KToolBarPrivate::slotContextTextRight()
     toolButtonStyleSettings[Level_UserSettings] = q->toolButtonStyle();
 }
 
-void KToolBarPrivate::slotContextIconSize()
+void KToolBarPrivate::slotContextIconSize(QAction *action)
 {
-    QAction *action = qobject_cast<QAction *>(q->sender());
-    if (action && contextIconSizes.contains(action)) {
-        const int iconSize = contextIconSizes.value(action);
-        q->setIconDimensions(iconSize);
+    if (action) {
+        auto it = std::find_if(m_contextIconSizes.cbegin(), m_contextIconSizes.cend(), [action](const ContextIconInfo &info) {
+            return info.iconAction == action;
+        });
+        if (it != m_contextIconSizes.cend()) {
+            q->setIconDimensions(it->iconSize);
+        }
     }
 }
 
@@ -887,20 +915,6 @@ KToolBar::~KToolBar()
     delete d->contextLockAction;
 }
 
-#if KXMLGUI_BUILD_DEPRECATED_SINCE(5, 0)
-void KToolBar::setContextMenuEnabled(bool enable)
-{
-    d->enableContext = enable;
-}
-#endif
-
-#if KXMLGUI_BUILD_DEPRECATED_SINCE(5, 0)
-bool KToolBar::contextMenuEnabled() const
-{
-    return d->enableContext;
-}
-#endif
-
 void KToolBar::saveSettings(KConfigGroup &cg)
 {
     Q_ASSERT(!cg.name().isEmpty());
@@ -925,14 +939,6 @@ void KToolBar::saveSettings(KConfigGroup &cg)
     }
 }
 
-#if KXMLGUI_BUILD_DEPRECATED_SINCE(5, 0)
-void KToolBar::setXMLGUIClient(KXMLGUIClient *client)
-{
-    d->xmlguiClients.clear();
-    d->xmlguiClients << client;
-}
-#endif
-
 void KToolBar::addXMLGUIClient(KXMLGUIClient *client)
 {
     d->xmlguiClients << client;
@@ -945,8 +951,7 @@ void KToolBar::removeXMLGUIClient(KXMLGUIClient *client)
 
 void KToolBar::contextMenuEvent(QContextMenuEvent *event)
 {
-#if KXMLGUI_BUILD_DEPRECATED_SINCE(5, 0)
-    if (mainWindow() && d->enableContext) {
+    if (mainWindow()) {
         QPointer<KToolBar> guard(this);
         const QPoint globalPos = event->globalPos();
         d->contextMenu(globalPos)->exec(globalPos);
@@ -957,7 +962,6 @@ void KToolBar::contextMenuEvent(QContextMenuEvent *event)
         }
         return;
     }
-#endif
 
     QToolBar::contextMenuEvent(event);
 }
@@ -1151,7 +1155,7 @@ void KToolBar::dragEnterEvent(QDragEnterEvent *event)
         }
 
         if (!d->actionsBeingDragged.isEmpty()) {
-            QAction *overAction = actionAt(event->pos());
+            QAction *overAction = actionAt(event->position().toPoint());
 
             QFrame *dropIndicatorWidget = new QFrame(this);
             dropIndicatorWidget->resize(8, height() - 4);
@@ -1180,7 +1184,7 @@ void KToolBar::dragMoveEvent(QDragMoveEvent *event)
                 for (QAction *action : actions) {
                     // want to make it feel that half way across an action you're dropping on the other side of it
                     QWidget *widget = widgetForAction(action);
-                    if (event->pos().x() < widget->pos().x() + (widget->width() / 2)) {
+                    if (event->position().toPoint().x() < widget->pos().x() + (widget->width() / 2)) {
                         overAction = action;
                         break;
                     }
@@ -1252,9 +1256,9 @@ void KToolBar::dropEvent(QDropEvent *event)
 void KToolBar::mousePressEvent(QMouseEvent *event)
 {
     if (toolBarsEditable() && event->button() == Qt::LeftButton) {
-        if (QAction *action = actionAt(event->pos())) {
+        if (QAction *action = actionAt(event->position().toPoint())) {
             d->dragAction = action;
-            d->dragStartPosition = event->pos();
+            d->dragStartPosition = event->position().toPoint();
             event->accept();
             return;
         }
@@ -1270,7 +1274,7 @@ void KToolBar::mouseMoveEvent(QMouseEvent *event)
         return;
     }
 
-    if ((event->pos() - d->dragStartPosition).manhattanLength() < QApplication::startDragDistance()) {
+    if ((event->position().toPoint() - d->dragStartPosition).manhattanLength() < QApplication::startDragDistance()) {
         event->accept();
         return;
     }
@@ -1326,7 +1330,8 @@ bool KToolBar::eventFilter(QObject *watched, QEvent *event)
         if (me->buttons() & Qt::RightButton) {
             if (QWidget *ww = qobject_cast<QWidget *>(watched)) {
                 if (ww->parent() == this && !ww->isEnabled()) {
-                    QCoreApplication::postEvent(this, new QContextMenuEvent(QContextMenuEvent::Mouse, me->pos(), me->globalPos()));
+                    QCoreApplication::postEvent(this,
+                                                new QContextMenuEvent(QContextMenuEvent::Mouse, me->position().toPoint(), me->globalPosition().toPoint()));
                 }
             }
         }
@@ -1352,19 +1357,34 @@ bool KToolBar::eventFilter(QObject *watched, QEvent *event)
             switch (event->type()) {
             case QEvent::MouseButtonPress: {
                 QMouseEvent *me = static_cast<QMouseEvent *>(event);
-                QMouseEvent newEvent(me->type(), mapFromGlobal(ww->mapToGlobal(me->pos())), me->globalPos(), me->button(), me->buttons(), me->modifiers());
+                QMouseEvent newEvent(me->type(),
+                                     mapFromGlobal(ww->mapToGlobal(me->position().toPoint())),
+                                     me->globalPosition().toPoint(),
+                                     me->button(),
+                                     me->buttons(),
+                                     me->modifiers());
                 mousePressEvent(&newEvent);
                 return true;
             }
             case QEvent::MouseMove: {
                 QMouseEvent *me = static_cast<QMouseEvent *>(event);
-                QMouseEvent newEvent(me->type(), mapFromGlobal(ww->mapToGlobal(me->pos())), me->globalPos(), me->button(), me->buttons(), me->modifiers());
+                QMouseEvent newEvent(me->type(),
+                                     mapFromGlobal(ww->mapToGlobal(me->position().toPoint())),
+                                     me->globalPosition().toPoint(),
+                                     me->button(),
+                                     me->buttons(),
+                                     me->modifiers());
                 mouseMoveEvent(&newEvent);
                 return true;
             }
             case QEvent::MouseButtonRelease: {
                 QMouseEvent *me = static_cast<QMouseEvent *>(event);
-                QMouseEvent newEvent(me->type(), mapFromGlobal(ww->mapToGlobal(me->pos())), me->globalPos(), me->button(), me->buttons(), me->modifiers());
+                QMouseEvent newEvent(me->type(),
+                                     mapFromGlobal(ww->mapToGlobal(me->position().toPoint())),
+                                     me->globalPosition().toPoint(),
+                                     me->button(),
+                                     me->buttons(),
+                                     me->modifiers());
                 mouseReleaseEvent(&newEvent);
                 return true;
             }
